@@ -57,13 +57,3 @@ def _annotated_protos_for(fn):
         args = []
 
     return [arg for arg in args if inspect.isclass(arg) and issubclass(arg, Message) and arg != Any]
-
-
-def _extend_args(args, task_args):
-    if not any(task_args):
-        return args
-
-    if not _is_tuple(args):
-        args = (args,)
-
-    return args + task_args
\ No newline at end of file
